@@ -1,10 +1,31 @@
-
 #       dbadd - add, delete objects
 #
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: dbadd.pl,v 2.6 1998/07/15 18:01:24 joao Exp $
+#
 #	$RCSfile: dbadd.pl,v $
-#	$Revision: 2.2 $
-#	$Author: ripe-dbm $
-#	$Date: 1996/08/08 19:30:08 $
+#	$Revision: 2.6 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:24 $
 
 require "dbopen.pl";
 require "dbclose.pl";
@@ -19,6 +40,7 @@ require "enread.pl";
 require "encmp.pl";
 require "updatecheck.pl";
 require "serial.pl";
+require "cross-notify.pl";
 
 sub finalchecksandupdate {
     local(*entry, $type, *autonichandles, *options)=@_;
@@ -273,7 +295,7 @@ sub finalchecksandupdate {
           }
                                              
        }
-       else {
+       else {         # $options & $DELETEOPTION
        
           local(@uniquekey)=(&enukey(*entry, $type));
           local(@result)=&dbmatch(*db, *uniquekey, "", 0);
@@ -510,41 +532,54 @@ sub finalchecksandupdate {
              
              # print STDERR "trouve! do check mnt: ($returncode)",$entry{"mb"}," ",$entry{"ml"},"\n";
              
-             local($notfound)="";
-          
-             foreach (split(/\s+/, $entry{"mb"}." ".$entry{"ml"})) {
+             my @notfound=();
+
+	     my $mntner;
+             my @mntners = undef;
+	     push @mntners, split(' ', $entry{mb}),
+ 	                    split(' ', $entry{ml});
+
+	     ## foreach was giving errors
+	     ## 2nd time round the loop $mntner was set to 1
+
+	     while ($mntner = shift @mntners) {
                 
-                # print STDERR "mt: $_\n";
+	       print STDERR "mt: $mntner\n" if $opt_V;
                 
-                next if ((!$_) ||
-                         ($_ eq $entry{"mt"}) ||
-                         ($ExistMaintainer{$_}) ||
-		         (&GetMaintainer($_, $entry{"so"}, 0)));
+	       next if !$mntner;
+	       next if ($mntner eq $entry{"mt"});
+	       next if ($ExistMaintainer{$mntner});
+	       next if (&GetMaintainer($mntner, $entry{"so"}, 0));
                 
-                $notfound.=$_." ";
-             
+	       push @notfound, $mntner;
              }
              
-             if ($notfound) {
-                
-                chop($notfound);
-                
-                &adderror(*entry, "unknown maintainer(s) \"$notfound\" referenced");
+             if (@notfound) {
+                &adderror(*entry,
+			  "unknown maintainer(s) \"@notfound\" referenced");
                 $returncode=$GENERALERROR;
-                
              }
-             
           }
        
-          $returncode=&updatecheck(*db, *oldobject, *entry, $type, $options) if ($returncode==$O_OK);
-          
+	  if ($returncode==$O_OK) {
+	    $returncode=&updatecheck(*db, *oldobject, *entry, $type, $options);
+          }
+
           if ($returncode==$O_OK) { 
 
-	     $returncode=&dbdel(*db, *oldobject, $type, $options | $NOCHECKSOPTION) if ($options & $MODIFYOPTION);
-	     $returncode=&dbadd(*db, *entry, $type, $options) if ($returncode==$O_OK);
+	       if ($options & $MODIFYOPTION) {
+		 $returncode=&dbdel(*db, *oldobject, $type,
+				    $options | $NOCHECKSOPTION);
+	       }
+
+	       if ($returncode==$O_OK) {
+		 $returncode=&dbadd(*db, *entry, $type, $options) ;
+	       }
 	  
-	     &AddNotify(*oldobject, *entry) if ($returncode==$O_OK);
+	     if ($returncode==$O_OK) {
+	       &AddNotify(*oldobject, *entry);
 	
+	     }
           }
        
        }
@@ -554,6 +589,7 @@ sub finalchecksandupdate {
        &dbclose(*db);
        &dbclose(*othernicdb) if (($split) && (($type eq "ro") || ($type eq "pn")));
     
+
     }
     else {
     
@@ -563,9 +599,37 @@ sub finalchecksandupdate {
        
     }
     
+
+	   # If object has been successfully updated ($O_OK ?)  do the
+	   # cross notification checks and generate messages if
+	   # needed.  Messages will be in a form suitable to feed to
+	   # to sendmail (or other MTA) and are put in temporary
+	   # files. add_cross_notify returns (possibly empty) list of
+	   # file names.  They will be sent at the same time as the
+	   # Update Notification.
+
+    # Do the cross notification checks here
+    #   1. after the dbases have been closed (we need to open them again)
+    #   2. while the new and old objects are still in scope
+
+    # use $::REPLYTO as the originator of the update message
+    # it gets set somewhere (?)
+
+    undef @::cross_notification_mailfiles;
+
+    if ($returncode == $O_OK) {
+      if ($options & $DELETEOPTION) {
+	push @::cross_notification_mailfiles,
+          add_cross_notify(\%entry, undef, $::REPLYTO);
+      }
+
+      if ($options & $NEWOPTION) {
+	push @::cross_notification_mailfiles,
+          add_cross_notify(undef, \%entry, $::REPLYTO);
+      }
+    }
+
     return ($returncode, "");
-
-
 }
 
 sub dbadd {
