@@ -1,13 +1,34 @@
-#
 # dbopen - open a database and lock exclusively if opened for writing
 #
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: dbopen.pl,v 2.10 1998/07/15 18:01:25 joao Exp $
+#
 # ***NOTE***: flock doesn't work on NFS mounted file systems!!!
 #             and thus locking doesn't work properly on NFS file systems
 #
 #	$RCSfile: dbopen.pl,v $
-#	$Revision: 2.2 $
-#	$Author: david $
-#	$Date: 1996/08/16 13:00:34 $
+#	$Revision: 2.10 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:25 $
 #
 #
 # arguments:
@@ -18,6 +39,12 @@
 
 require "defines.pl";
 
+# return name of db file given text-file name
+sub dbfilename { return $_[0].'.db' }
+
+use Fcntl;
+use DB_File;
+
 sub dbname {
     local(*entry)=@_;
     
@@ -56,6 +83,7 @@ sub dbopen {
 	    sleep 60;
 	}
 	
+	# Danger! Cleaning index could create lockfile here !
 	
         if (!open($database, "+<$name")) {
            
@@ -109,10 +137,22 @@ sub dbopen {
     local($oldhandle)=select($database); $|=1; select($oldhandle);
     
     if ($write) {
-       dbmopen(%database, $name, 0644) || &fatalerror("Couldn\'t open dbm file($!): $name");  
+      my($dbfilename) = dbfilename($name);
+
+# Eek! this should be in the configuration file
+
+#      my($a) = new DB_File::HASHINFO;
+#      $a->{cachesize} = 32*1024*1024;
+
+      $DB_HASH->{cachesize} = $DBCACHESIZE;
+
+      tie (%database, DB_File, $dbfilename, O_CREAT|O_RDWR, 0644, $DB_HASH )
+	|| &fatalerror("Couldn\'t open dbm file($!): $dbfilename");  
     }
     else {
-       dbmopen(%database, $name, undef) || return 0;
+      my($dbfilename) = dbfilename($name);
+
+      tie (%database, DB_File, $dbfilename, O_RDONLY) || return 0;
     }
     
     @database=(1, $name);
@@ -143,11 +183,21 @@ sub dbclopen {
     $name=&dbname(*entry) if ($name=~ /^\s*$/);
     $name.=$CLASSLESSEXT;
 
+    my($dbfilename)=dbfilename($name);
+
     if ($write) {
-       dbmopen(%mspnxl, $name, 0644) ||  &fatalerror("Couldn\'t open classless index ($name) for writing: $!");
+#      my($a) = new DB_File::HASHINFO;
+#      $a->{cachesize} = 32*1024*1024;
+
+      $DB_HASH->{cachesize} = $DBCACHESIZE;
+
+       tie (%mspnxl, DB_File, $dbfilename, O_CREAT|O_RDWR, 0644, $DB_HASH) ||
+	 &fatalerror("Couldn\'t open classless index ($dbfilename) for writing: $!");
+#       dbmopen(%mspnxl, $name.$DBFileExt, 0644) ||  &fatalerror("Couldn\'t open classless index ($name$DBFileExt) for writing: $!");
     }
     else {
-       dbmopen(%mspnxl, $name, undef) || return 0;
+#       dbmopen(%mspnxl, $name.$DBFileExt, undef) || return 0;
+       tie (%mspnxl, DB_File, $dbfilename, O_RDONLY) || return 0;
     }
     
     @mspnxl=(1, $name);
