@@ -1,11 +1,32 @@
-
 #       encmp - compare two associative arrays
 #               'syntax sugar' is removed before comparing!
 #
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: encmp.pl,v 2.4 1998/07/15 18:01:27 joao Exp $
+#
 #	$RCSfile: encmp.pl,v $
-#	$Revision: 2.1 $
-#	$Author: ripe-dbm $
-#	$Date: 1996/08/14 15:45:19 $
+#	$Revision: 2.4 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:27 $
 #
 #	This routine takes two objects and compares them case insensitive
 #	Returns 1 if objects are equal, else returns 0;
@@ -38,6 +59,7 @@ sub encmp {
         $a=$this{$_};
 	$b=$that{$_};
         
+	next if ($_ eq "ch");
         # print STDERR "-$a-$b-";
         
 	# we need to do quite a bit of work for comparing attributes
