@@ -1,14 +1,40 @@
-
 #       misc - miscellaneaous functions
 #
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: misc.pl,v 2.14 1998/07/15 18:01:33 joao Exp $
+#
 #	$RCSfile: misc.pl,v $
-#	$Revision: 2.1 $
-#	$Author: ripe-dbm $
-#	$Date: 1996/08/14 15:46:06 $
+#	$Revision: 2.14 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:33 $
 #
 
 require "defines.pl";
 
+# sort block for sorting changed fields
+# needs to cope with 6 or 8 digit dates (YYMMDD or YYYYMMDD)
+# regexp is tied with '$' so they will match 6 LSB digits and ignore
+# century part.
+
 sub sortchangedfield {
     
     $a=~ /(\d{6,6})$/;
@@ -249,7 +275,48 @@ sub iprightzeromask {
     #
     # ipv4 version
     
-    return $ipvalue & $masks[$length] if ($ipvalue!~ /\:/);
+    if ($ipvalue!~ /\:/) {
+
+      # This bit can cause problems. Some versions of perl return
+      # a signed result which causes classless lookups to fail.
+
+      # check from the command line with something like...
+
+      # perl5.00307  -e 'print 3238002689 & 4294967040, "\n"'
+      # -1056964608
+      # perl5.00397  -e 'print 3238002689 & 4294967040, "\n"'
+      # 3238002688
+      # perl5.00401  -e 'print 3238002689 & 4294967040, "\n"'
+      # 3238002688
+
+      # the first one is broken.
+
+      # But even 5.00401 doesn't completely fix it. Perl will
+      # convert between numbers and strings automagically,
+      # but something going wrong with bitwise &.
+      #
+      # perl5.00401  -e 'print "3238002689" & 4294967040, "\n"'
+      # 2147483392
+
+      # We force $ipvalue to be numeric (rather than a string of digits)
+      # by decrementing and incrementing it.
+      # (inc then dec if it is zero so it doesn't become signed)
+
+      if ($ipvalue != 0) {
+        $ipvalue--; $ipvalue++;
+      } else {
+        $ipvalue++; $ipvalue--;
+      }
+
+      {
+	my ($result) = $ipvalue & $masks[$length];
+
+#	my ($quad) = int2quad($result);
+#	print "iprightzeromask: [$result] [$quad]\n" if $opt_V;
+
+	return $result; 
+      }
+    }
     
     #
     # now try the ipv6 version   
@@ -565,12 +632,67 @@ sub isnichandle {
     #    - country codes are allowed for APNIC compatibility
     #    - APNIC AP|CC-AU exceptions are handled correctly
     #    - -ORG organization InterNIC handles
+    #    - -ARIN ARIN handles
+    #    - -ORG-ARIN ARIN handles
     
-    return scalar(($_[0]!~ /^AUTO$/i) &&
-                  (($_[0]=~ /^[A-Z]{2,$MAXLENGTHINITIALS}([1-9]\d{0,5})?(\-$SOURCENAME|\-[12]\-AU)?$/o) ||
-                   ($_[0]=~ /^[A-Z]{2,$MAXLENGTHINITIALS}(\d{0,4})JP(\-JP)?$/o)) &&
-                  (($2!~ /^\-(\S+)$/) || ($DBFILE{$1}) || ($1 eq "ORG") || ($COUNTRY{$1}) || ($1=~ /^AP|CC-AU|[12]\-AU$/)));
-    
+  my $handle = shift;
+  
+  # reject lowercase
+  return undef if $handle =~ /[a-z]/;
+
+  # Japanese NIC handles
+  # 
+  # leading zeros in the number part *are* allowed
+  #
+  # e.g. AB021JP AB199JP-JP
+  #
+  return 1 if ($handle =~ /[A-Z]{2}\d{3}JP(-JP)?/);
+
+  # Standard NIC handles
+  #
+  # leading zeros in the number part are *not* allowed
+  #
+  # InterNIC - TBQ, IP4
+  # RIPE format - AB1-RIPE
+  # APNIC use two letter country code suffix
+  # Austraila have used -1-AU, -2-AU, -CC-AU suffix.
+  # Internic used -ORG suffix
+  # ARIN use -ARIN suffix
+  # ARIN also use -ORG-ARIN suffix
+  #
+  
+  if ($handle =~ /^([A-Z]{2,$MAXLENGTHINITIALS})([1-9]\d{0,5})?(-\S+)?$/) {
+    my ($initials,$number,$suffix) = ($1,$2,$3);
+
+    # we're done if there is no suffix
+    return 1 unless $suffix;
+
+    # strip leading '-'
+    $suffix =~ s/^-//;
+
+    # suffix of local sources (better to use the reverse hash of NICPOSTFIX)
+    foreach $v (values %NICPOSTFIX) {
+      return 1 if ($suffix eq $v);
+    }
+
+#    return 1 if (defined $NICPOSTFIX{$suffix});
+
+    # country codes
+    return 1 if (defined $COUNTRY{$suffix});
+
+    # special suffix (using a 'static' hash would be quicker)
+    return 1 if (
+		 ($suffix eq 'CC-AU') ||
+		 ($suffix eq '1-AU') ||
+		 ($suffix eq '2-AU') ||
+		 ($suffix eq 'ORG') ||
+                 ($suffix eq 'ARIN') ||
+                 ($suffix eq 'ORG-ARIN') ||
+		 ($suffix eq 'AP')
+		 );
+  }
+
+  return undef;
 }
 
 
@@ -895,4 +1017,31 @@ sub unlock {
 
     return (flock($file, $LOCK_UN));
 }
+
+# Provides a simple wrapper around the setpriority or renice command. Use
+# setpriority syscall unless RENICECMD is defined in config file.
+
+# $RENICECMD is set from config file in rconf.pl
+
+{
+
+  # value to change priority to (should probably be in config file)
+  my($nice_value)=10;
+
+  # lower priority of running process
+  # takes no args - returns status of syscall or system
+  #
+
+    sub lower_priority {
+
+      if (defined $RENICECMD) {
+	print STDERR "using [$RENICECMD]\n" if $opt_V;
+	return system("$RENICECMD 10 $$ > /dev/null 2>/dev/null");
+      } else {
+	print STDERR "using setpriority(2)\n" if $opt_V;
+	return setpriority PRIO_PROCESS, 0, $nice_value;
+      }
+    }
+}
+
 1;
