@@ -1,8 +1,31 @@
+# net2net.pl
+#
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: net2net.pl,v 2.7 1998/07/15 18:01:34 joao Exp $
 #
 #	$RCSfile: net2net.pl,v $
-#	$Revision: 2.0 $
-#	$Author: ripe-dbm $
-#	$Date: 1996/08/08 10:47:30 $
+#	$Revision: 2.7 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:34 $
 #
 
 require "defines.pl";
@@ -21,6 +44,12 @@ require "misc.pl";
 #
 # David K. 950908
 
+#
+# replaced complicated regexp to something more understandable.
+# int2quad and quad2int can be done simply with pack/unpack
+#
+# Chris
+
 #
 # quad quad2int(int, option)
 #
@@ -28,33 +57,52 @@ require "misc.pl";
 # option=1   - log errors
 
 sub quad2int {
+  my($string, $log) = @_;
 
-    return($4 + $3*256 + $2*65536 + $1*16777216) if ($_[0]=~ /^0*(1?[1-9]?\d|[12][0-4]\d|25[0-5])\.0*(1?[1-9]?\d|[12][0-4]\d|25[0-5])\.0*(1?[1-9]?\d|[12][0-4]\d|25[0-5])\.0*(1?[1-9]?\d|[12][0-4]\d|25[0-5])$/);
-       
-    &syslog("ERRLOG", "quad2int: illegal ip number \"".$_[0]."\"") if ($_[1]);
+  my(@bytes) = split(/\./, $string);
+
+  # check it looks like an ipv4 address.
+  # there should be four parts, each being all digits with a value less
+  # than 256
+  #
+  if (@bytes == 4 && scalar(grep {/^\d+$/ && $_<256} @bytes) == 4) {
+    return unpack(N,pack(C4,@bytes));
+  }
+
+  &syslog("ERRLOG", "quad2int: illegal ip number [$string]") if ($log);
        	   
-    return(-1);
-    
+  return(-1);
 }
 
 #
 # int int2quad(quad)
 #
-# I expected the need for a very tiny addition (smaller then 1/16777216 
-# to avoid other problems) to avoid nasty rounding off errors but 
-# it looks like perl *does* do integer divisions contrary to the book...
-#
-# David K.
+# given (long) integer returns a string representing it as an IP address style
+# dotted quad.
 
 sub int2quad {
-    
-    return join(".", int($_[0]/16777216) & 0xff,
-                     int($_[0]/65536) & 0xff,
-                     int($_[0]/256) & 0xff,
-                     $_[0] & 0xff);
-    
+  use strict;
+
+  my($int)=@_;
+
+  # nasty hack to make perl treat the integer as a numeric rather than a 
+  # string. (see also iprightzeromask in misc.pl)
+  #
+  # just do a simple arithmetic operation on it - decrement then increment
+  # unless it's zero.
+
+  if ($int == 0) {
+    $int++; $int--;
+  } else {
+    $int--; $int++;
+  }
+
+  return join('.', unpack('C4',pack('N',$int)));
 }
 
+# convert range in form 'a.b.c.d - a.b.255.255'
+# to list of prefixes in form 'n/p' where n is 32 bit int IP address
+# p is prefix.
 
 sub range2prefixes {
    
@@ -64,29 +112,34 @@ sub range2prefixes {
     $ip1=&quad2int($ip1,0);
     $ip2=&quad2int($ip2,0);
    
-    local($bits);
    
-    for (;;) {
+    do {
    
-       for ($bits=32-int(log($ip2-$ip1+1)*$ONEDIVLOG2+$TINYLOGVALUE);
-            $ip1!=($ip1 & $masks[$bits]);
-            $bits++) {};
+      # get smallest useful prefix length based on size of range
+      local($bits) = 32-int(log($ip2-$ip1+1)*$ONEDIVLOG2+$TINYLOGVALUE);
+
+      {
+	use integer;       # for the following bitwise &
+
+	# find smallest prefix that masks all bits of IP address
+	until ($ip1 == ($ip1 & $masks[$bits])) {
+	  $bits++;
+	}
+      }
    
-       push(@prefixes, $ip1."/".$bits);
+      push(@prefixes, $ip1."/".$bits);
        
-       last if (($ip1=$ip1+(2**(32-$bits))-1)==$ip2);
-       
-       $ip1++;
-          
-    }
+      # add on block we've just found
+      $ip1 += 2**(32-$bits); 
+
+      # stop when we've reached the top end of range
+    } until (($ip1-1) == $ip2);
+
    
     return @prefixes;
-   
 }
 
 
-
-
 sub completeip {
     local($range)=@_;
     
@@ -132,6 +185,8 @@ sub normalizerange {
     
     if ($range=~ /^(\d+)(\.[\.\d]*)? *([\/\-]) *([\d\.]+)*$/) {
        
+      # Reject 0/8, 10/8, 127/8 & 224/4 networks
+
        return ("", $O_INVALIDPREFIX) if (($1=~ /^0*1?0|127$/) || ($1>223)); 
        
        if ($3 eq "\/") {
