@@ -1,8 +1,31 @@
-
+# syntax.pl
+#
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: syntax.pl,v 2.11 1998/07/15 18:01:41 joao Exp $
+#
 #	$RCSfile: syntax.pl,v $
-#	$Revision: 2.4 $
-#	$Author: david $
-#	$Date: 1996/08/16 13:00:34 $
+#	$Revision: 2.11 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:41 $
 #
 #       ARGUMENTS:      *ASSOC object
 #       RETURNS:        INTEGER object_status
@@ -468,12 +491,20 @@ sub dosyntax {
 #
 # Now check the actual keywords
 #
-	while($tmppol =~ s/(\S+)//) {
-	    if (!&isaskeyword($1)) {
-		return $O_ERROR, 
-		"syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n\t$1 ".
+# Joao. Including Curtis Villamizar's performance patch.
+#	while($tmppol =~ s/(\S+)//) {
+#	    if (!&isaskeyword($1)) {
+#		return $O_ERROR, 
+#		"syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n\t$1 ".
+#
+#		    "is not a routing policy KEYWORD";
+        local($tmpword);
+        foreach $tmpword ( split(/\s+/, $tmppol) ) {
+ 	   if (!&isaskeyword($tmpword)) {
+ 	       return $O_ERROR, 
+ 	       "syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n".
+ 		   "\t$tmpword is not a routing policy KEYWORD";
 
-		    "is not a routing policy KEYWORD";
 	    }
 	}
 	return $O_OK;
@@ -618,12 +649,17 @@ sub dosyntax {
 	elsif ($authstr[0] eq "MAIL-FROM") {
 	    local($regex) = $value;
 	    $regex =~ s/^\s*MAIL\-FROM\s*//;
-	    eval "/$regex/;";
+
+	    $regex =~ s/\\?\@/\Q\@\E/g;   # quote '@'s for perl5
+
+	    # Check the given regexp is valid.
+	    eval { m/$regex/ };
+
 	    if ($@) {
-		return $O_ERROR, "\"$regex\" is not a legal regular expression";
+	      return $O_ERROR, "\"$regex\" is not a legal regular expression";
 	    }
 	    else {
-		return $O_OK;
+	      return $O_OK;
 	    }
 	}
 	    
@@ -726,7 +762,8 @@ sub dosyntax {
        #
        # add current date if needed to last specified attribute
        
-       if ($changedvalues[$#changedvalues]!~ / \d{6,6}$/) {
+       # match 6 or 8 digits YYMMDD or YYYYMMDD
+       if ($changedvalues[$#changedvalues]!~ / (\d{2}){3,4}$/) {
           
           $changedvalues[$#changedvalues].=" ".$DATE;
           
@@ -762,28 +799,36 @@ sub dosyntax {
 	     return $O_ERROR, "No date specified in \"$ATTL{$key}\" attribute";
           }
            
-	  if ($date=~ /^(\d\d)(\d\d)(\d\d)$/) {
-	  
-	     #
-	     # 1988 is the start of the world. This is where we test for proper
-	     # date values of YYMMDD
+	  if ($date=~ /^(\d{2}|\d{4})(\d\d)(\d\d)$/) {
+	    
+	    local($year,$month,$day) = ($1,$2,$3);
 
-             if ($1<88) {
-	        return $O_ERROR, "date ($date) part of \"$ATTL{$key}\" is older then the database itself!";
-	     }
+	    $year += 1900  if ($1<100);  # add century part if needed
 
-	     if (($2<1) || ($2>12) || ($3>31) || ($3<1)) {
-	        return $O_ERROR, "date ($date) part of \"$ATTL{$key}\" is not a valid YYMMDD value";
-	     }
+	    if ($year<1988) {
+	      return $O_ERROR,
+	        "date ($year in $date) part of \"$ATTL{$key}\" is older than the database itself! (1988)";
+	    }
+
+	    if (($month<1) || ($month>12)) {
+	      return $O_ERROR,
+	        "Month part ($month) of \"$ATTL{$key}\" is not valid";
+	    }
 	  
+	    if (($day>31) || ($day<1)) {
+	      return $O_ERROR,
+	        "Day part ($day) of \"$ATTL{$key}\" is not valid";
+	    }
+
 	  }
 	  else {
 	  
-	     return $O_ERROR, "date part of \"$ATTL{$key}\" not in YYMMDD format";
+	     return $O_ERROR, "date part of \"$ATTL{$key}\" not in YYMMDD or YYYYMMDD format";
 	
 	  }
 	  
-	  if ($date>$DATE) {
+	  if (((length($date)==6) && ($date > $DATE)) ||
+	      ((length($date)==8) && ($date > ('19'.$DATE)))) {
 	     
 	     &addwarning(*object, "date in \"$ATTL{$key}\" ($date) is in the future - changed to $DATE");
 	     $date=$DATE;
@@ -839,6 +884,29 @@ sub dosyntax {
 	return $O_OK;
     }
     
+# cn - cross-nfy
+#
+# value is a nic handle
+
+    if ($key eq 'cn') {
+      return $O_OK if isnichandle($value);
+
+      return ($O_ERROR,
+	      "syntax error in \"$ATTL{$key}\" - Not a nic handle ($value)");
+    }
+
+
+# ct - cross-mnt
+#
+# value is name of a mntner
+
+    if ($key eq 'ct') {
+      return $O_OK if ismaintainer($value);
+
+      return ($O_ERROR,
+	      "syntax error in \"$ATTL{$key}\" - Not a maintainer ($value)");
+    }
+
 #
 # cy - country
 
@@ -1140,6 +1208,36 @@ sub dosyntax {
 	
     }
     
+#
+# rf - refer
+# referral attribute for domaon objects
+
+    if ($key eq "rf") {
+
+       print STDERR "dosyntax - checking rf\n" if $opt_V;
+
+	local($type,$host,$port) = split(/\s+/,$value);
+# Check syntax of TYPE field
+	if ($type !~ /^RIPE$/i && $type !~ /^internic$/i && $type !~ /^simple$/i) {
+	 return $O_ERROR,
+          "syntax error in \"$ATTL{$key}: $value\" - ".
+            "type must be one of RIPE, internic or simple";
+	}
+# Check syntax of host field
+       if (!&isdomname($host)) {
+          return $O_ERROR, "illegal host name in ".$ATTL{$key};
+       }
+
+# Check syntax of port field
+	if (defined $port && ($port !~ /[0-9]{1,5}/ || $port > 65534) ) {
+         return $O_ERROR, 
+          "syntax error in \"$ATTL{$key}: $value\" - ".
+	  "port must be numeric value in the range 0-65534";
+	}
+
+	return $O_OK;
+}
+	   
 #
 # do - dom-out
 #
