@@ -1,10 +1,31 @@
-
 #       whoisd - whois Internet daemon
 #
+# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
+#
+# All Rights Reserved
+#
+# Permission to use, copy, modify, and distribute this software and its
+# documentation for any purpose and without fee is hereby granted,
+# provided that the above copyright notice appear in all copies and that
+# both that copyright notice and this permission notice appear in
+# supporting documentation, and that the name of the author not be
+# used in advertising or publicity pertaining to distribution of the
+# software without specific, written prior permission.
+#
+# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
+# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
+# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+#
+# $Id: whoisd.pl,v 2.21 1998/07/15 18:01:44 joao Exp $
+#
 #	$RCSfile: whoisd.pl,v $
-#	$Revision: 2.3 $
-#	$Author: ambrose $
-#	$Date: 1996/11/08 09:53:39 $
+#	$Revision: 2.21 $
+#	$Author: joao $
+#	$Date: 1998/07/15 18:01:44 $
 #
 
 #
@@ -33,15 +54,11 @@
 # HUP  reread config file
 
 
-PERL5OPTIONS
+use Socket;
+#use POSIX qw/WNOHANG/;   # used by waitpid
 
 require "flush.pl";
 
-$SOL_SOCKET = 0xffff;
-$SO_REUSEADDR = 0x0004;
-
-@INC = ("LIBDIR", @INC);
-
 require "rconf.pl";
 
 require "dbopen.pl";
@@ -61,6 +78,10 @@ require "template.pl";
 require "whoisqry.pl";
 require "processdata.pl";
 
+require "read_attr.pl";
+
+require "read_objs.pl";
+
 #
 # If we get a SIGALRM, exit. Used in the read loop, to make sure people
 # do not keep the connection open, and open and open ....
@@ -484,6 +505,38 @@ sub lookupandprint {
     return $printed;
     
 }
+#
+# Error and help message for "-t" and "-v" options to whois
+
+sub templaerror {
+	my($templatypes, $output) = @_;
+
+	  print $output "\n% $templatypes are available for these objects:\n\n";
+	  foreach $object (sort keys (%OBJATSQ)) {
+		printf $output "%%\t\t$ATTL{$object}\n"; 
+		}
+	} # End of sub templaerror
+
+sub verbose_error {
+	my($output) = @_;
+
+	print $output "\nUse\n";
+	print $output "\nwhois -v <object-type>\n";
+	print $output "\nto obtain detailed information about RIPE database objects\n";
+	print $output "where <object-type> is one of\n\n";
+
+	foreach $object (sort keys (%OBJATSQ)) {
+		printf $output "$ATTL{$object}\n\n";
+		printf $output "$obj_descs{$ATTL{$object}}\n\n\n";
+	} # End of foreach statement
+
+	printf $output "\n\nFurther information may be found at:\n\n";
+	printf $output "<http://www.ripe.net/docs/ripe-157.html>\n";
+	printf $output "<ftp://ftp.ripe.net/ripe/docs/ripe-157.ps>\n";
+	printf $output "<ftp://ftp.ripe.net/ripe/docs/ripe-157.txt>\n";
+
+	} # End of verbose_error
+
 
 #
 # whois - main lookup loop. will output all objects found for all sources
@@ -501,17 +554,41 @@ sub whois {
        alarm $KEEPOPEN;
        
        if ($OBJATSQ{$opt_t}) {
-          &Template($output,$opt_t);
+          &Template($output, $opt_t);
        }
-       else {
-# Patch from David, 961010, replaced this; AMRM, 961010
-#          print $output "\n% No template available for object \"$opt_t\"";
+       elsif ($opt_t =~ /^all$/i) {
+	&templaerror("Templates", $output);
+	}
+	else 	{
           print $output "\n% No template available for object \"$opt_t\"\n";
+	&templaerror("Templates", $output);
        }
 
        return 1;
 
     }
+
+    if ($opt_v) {
+       
+       $opt_v = $ATTR{$opt_v} if $ATTR{$opt_v};
+
+       alarm $KEEPOPEN;
+       
+       if ($OBJATSQ{$opt_v}) {
+          &Verbose($output,$opt_v);
+       }
+       elsif ($opt_v =~ /^all$/i) {
+	&verbose_error($output);
+	}
+       else {
+          print $output "\n% No verbose template available for object \"$opt_v\"\n";
+	  &verbose_error($output);
+       }
+
+       return 1;
+
+    }
+
     elsif ($opt_g) {
        
        if ($opt_D) {
@@ -524,7 +601,14 @@ sub whois {
        
     }
     elsif ($searchstring=~ /^\s*(HELP|HOWTO)\s*$/i) {
-       open (HELP, $HELP);
+       open (HELP, $WHOISHELP);
+       alarm $KEEPOPEN;
+       print $output <HELP>, "\n";
+       close(HELP);
+       return 1;
+    }
+    elsif ($searchstring=~ /^\s*(AUTO[\-\s]*DBM(HELP)?)\s*$/i) {
+       open (HELP, $AUTODBMHELP);
        alarm $KEEPOPEN;
        print $output <HELP>, "\n";
        close(HELP);
@@ -644,8 +728,13 @@ sub whois {
           # print STDERR "keys: ", join(" ", keys(%OBJATSQ)), "\n";
           # print STDERR "oldorder: ", join(" ", keys(%OBJATSQ)), "\n";
           # print STDERR "order: ", join(" ", sort referenceorder keys(%OBJATSQ)), "\n";
-         
-          foreach $j (sort referenceorder @searchdb) {
+
+	  my(@sorted) = sort referenceorder @searchdb;
+
+	  # this was 'foreach $j (@sorted)' but in the final iteration
+	  # $j was always '1'. Couldn't find the bug so worked around
+	  #
+          while($j = shift @sorted) {
              
              $dbfile=$DBFILE{$source};
              $dbfile.=".".$j if ($SPLIT{$source});    
@@ -676,8 +765,26 @@ sub whois {
                    
                    foreach (grep($OBJATSQ{$j}=~ /(^| )$_( |$)/, split(/ /, $opt_i))) {
                       @keys=&makekeys($searchstring, $_, *longkeys);
-                      &lookupandprint($output, *whoisdb, *longkeys, *references, $opt_T, *result, $opt_F | $opt_L | $opt_m | $opt_M | $opt_r | $opt_S | $EXACTMATCHOPTION) ||
-                      &lookupandprint($output, *whoisdb, *keys, *references, $opt_T, *result, $opt_F | $opt_L | $opt_m | $opt_M | $opt_r | $opt_S | $EXACTMATCHOPTION);
+                      &lookupandprint(
+				      $output,
+				      *whoisdb,
+				      *longkeys,
+				      *references,
+				      $opt_T,
+				      *result,
+				      $opt_F | $opt_L | $opt_m | $opt_M |
+				      $opt_r | $opt_S | $EXACTMATCHOPTION
+				      )
+			|| &lookupandprint(
+					   $output,
+					   *whoisdb,
+					   *keys,
+					   *references,
+					   $opt_T,
+					   *result,
+					   $opt_F | $opt_L | $opt_m | $opt_M |
+					   $opt_r | $opt_S | $EXACTMATCHOPTION
+					   );
                    }
                 
                 }
@@ -740,7 +847,13 @@ sub whois {
           
           if (%references) {
           
-             foreach $type (keys %references) {
+	    # without the extra array (@types) the number of iterations
+	    # is cut short with the last value being '1'.
+	    # this is probably something to do with the deletion
+	    # of the entries of %references and the passing by type glob
+
+	    local(@types) = keys %references;
+	    foreach $type (@types) {
              
                 %doubles=();
                 
@@ -800,6 +913,7 @@ sub whois {
 sub clearflags {
 
     $opt_a=0;
+    $opt_c=0;
     $opt_g="";
     $opt_i="";
     $opt_F=0;
@@ -811,6 +925,7 @@ sub clearflags {
     $opt_S=0;
     $opt_t="";
     $opt_T="";
+    $opt_v="";
     $opt_Type=0;
     $opt_U="";
     $opt_Version="";
@@ -822,6 +937,7 @@ sub getflags {
     local(@flags)=();
     
     push(@flags, "a") if ($opt_a);
+    push(@flags, "c") if ($opt_c);
     push(@flags, "g") if ($opt_g);
     push(@flags, "i") if ($opt_i);
     push(@flags, "F") if ($opt_F);
@@ -833,6 +949,7 @@ sub getflags {
     push(@flags, "s") if ($opt_s);
     push(@flags, "S") if ($opt_S);
     push(@flags, "t") if ($opt_t);
+    push(@flags, "v") if ($opt_v);
     push(@flags, "T") if ($opt_Type);
     push(@flags, "U") if ($opt_U);
     push(@flags, "V".$opt_Version) if ($opt_Version);
@@ -862,11 +979,13 @@ sub parse {
     
     while ($string=~ /^-/) {
        
-       if ($string=~ s/^\-(\S*[arkFLmMS]+\S*) *//) {
+# Matches the part of the query string holding the options
+       if ($string=~ s/^\-(\S*[acrkFLmMS]+\S*) *//) {
           
           local($option)=$1;
           
           $opt_a=1 if ($option=~ s/a//g);
+          $opt_c=256 if ($option=~ s/c//g);
           $opt_r=$NONRECURSIVEOPTION if ($option=~ s/r//g);
           $opt_k=1 if ($option=~ s/k//g);
           $opt_F=$FASTOPTION if ($option=~ s/F//g);
@@ -1011,8 +1130,16 @@ sub parse {
             
         }
         
+        if ($string =~ s/^\-v +(\S+) *//)  {
+            $opt_v=$1;
+            
+            next;
+            
+        }
+
         last;
         
+
     }
     
     if (!$LOWPRIORITY) {
@@ -1027,8 +1154,9 @@ sub parse {
           
        }
     
-       system("$RENICECMD 10 $$ > /dev/null 2>/dev/null") if (($opt_k) || ($opt_m) || ($opt_M) || ($opt_g) || ($opt_U));
-    
+       if (($opt_k) || ($opt_m) || ($opt_M) || ($opt_g) || ($opt_U)) {
+	 lower_priority;
+       }
     }
     
     print STDERR "($$) left parse, searchstring: $string\n" if $opt_V;
@@ -1070,7 +1198,7 @@ sub parse {
           else {
                  
              alarm $KEEPOPEN;
-             print $output "\n% unknown attribute type \"$ATTL{$type}\" ignored\n";  
+             print $output "\n% unknown attribute type \"$type\" ignored\n";  
              &flush($output);
               
           }
@@ -1113,8 +1241,8 @@ sub parse {
        
        if ($opt_i=~ /^\s*$/) {
           $name.="(".$rhost.")" if ($name ne $rhost);
-          &syslog("QRYLOG","($$) [".&getflags()."] 0 $name no attributes specified for -i query");
-          &exitwhoisd(0, "% no attributes specified for -i query");  
+          &syslog("QRYLOG","($$) [".&getflags()."] 0 $name no valid attributes specified for -i query");
+          &exitwhoisd(0, "% no valid attributes specified for -i query");  
        }
        
     }
@@ -1423,7 +1551,7 @@ $opt_V);
             
       }
       
-      if (($query=~ /\w/) || ($opt_t) || ($opt_g) || ($opt_U)) {
+      if (($query=~ /\w/) || ($opt_t) || ($opt_v) || ($opt_g) || ($opt_U)) {
           
          ($result)=&whois($input, $output, $query, $name, $rhost);
          alarm $KEEPOPEN;
@@ -1509,7 +1637,9 @@ while ($arg=shift(@ARGV)) {
 # Read config file from RIPEDBCNF, or set to default.
 
 $conffile=$ENV{"RIPEDBCNF"};
-$conffile= "DEFCONFIG" unless $conffile;
+print STDERR "\$conffile is $conffile\n" if ($opt_V);
+$conffile= $RIPEConfig{DEFCONFIG} unless $conffile;
+print STDERR "\$conffile is $conffile\n" if ($opt_V);
 &rconf($conffile);
 
 $result=0;
@@ -1533,26 +1663,6 @@ if ((@ARGV) || (!$opt_D)) {
    exit 0;
    
 } 
-elsif (!$opt_V) {
-   
-   # 
-   # detach from tty
-
-   exit 0 if (fork()>0);
-
-   if (open(FILE, "/dev/tty")) {
-   
-      if (!ioctl(FILE,(0x20000000|(ord('t')<<8)|113),0)) {   
-         print STDERR "whoisd daemon ($$) - ioctl: $!\n" if ($opt_V);
-      }
-   
-      close(FILE);
-   
-   }
-   
-   close(STDIN) if (-t STDIN);
-   
-}
 
 #
 # we are running in daemon mode now
@@ -1571,7 +1681,7 @@ else {
 
 print STDERR "whoisd daemon ($$) - will connect to port: $port with protocol: $proto\n" if ($opt_V);
 
-socket(S, AF_INETVALUE, SOCK_STREAMVALUE, $proto) || &fatalerror("socket: $!");
+socket(S, PF_INET, SOCK_STREAM, $proto) || &fatalerror("socket: $!");
 
 #
 # || &fatalerror("setsockopt: $!"); commented out.
@@ -1579,19 +1689,25 @@ socket(S, AF_INETVALUE, SOCK_STREAMVALUE, $proto) || &fatalerror("socket: $!");
 # didn't work for perl4 & ( BSDI || Linux )
 #
 
-setsockopt(S, $SOL_SOCKET, $SO_REUSEADDR, 1); # || &fatalerror("setsockopt: $!");
+setsockopt(S, SOL_SOCKET, SO_REUSEADDR, 1); # || &fatalerror("setsockopt: $!");
 
-while (!bind(S, pack(SOCKADDRVALUE, AF_INETVALUE, $port, "\0\0\0\0"))) {
+while (!bind(S, sockaddr_in ($port, INADDR_ANY))) {
 
-    if ($bindcount>=20) {
-       print STDERR "whoisd: bind() failed 20 times, giving up\n";
-       &syslog("ERRLOG", "whoisd cannot bind() for 20 times, giving up");
+  # max number of times to try the bind
+  my $MAX = 2;
+
+  # number of seconds to wait between tries
+  my $WAIT = 7;
+
+    if ($bindcount>=$MAX) {
+       print STDERR "whoisd: bind() failed $MAX times, giving up\n";
+       &syslog("ERRLOG", "whoisd cannot bind() for $MAX times, giving up");
        exit 1;
     }
     else {
-       print STDERR "whoisd daemon ($$) - bind: $!, trying again after 7 seconds\n" if ($opt_V);
+       print STDERR "whoisd daemon ($$) - bind: $!, trying again after $WAIT seconds\n" if ($opt_V);
        $bindcount++;
-       sleep 7;
+       sleep $WAIT;
     }
 
 }
@@ -1617,6 +1733,31 @@ print STDERR "whoisd daemon ($$) - changed (UID,GID) to: ($>,$))\n" if ($opt_V);
 
 $oldhandle=select(S); $| = 1; select($oldhandle);
 
+# Now we're sure we can run 
+# detatch unless we're running verbose mode
+
+if (!$opt_V) {
+   
+   # 
+   # detach from tty
+
+   exit 0 if (fork()>0);
+
+   if (open(FILE, "/dev/tty")) {
+   
+      if (!ioctl(FILE,(0x20000000|(ord('t')<<8)|113),0)) {   
+         print STDERR "whoisd daemon ($$) - ioctl: $!\n" if ($opt_V);
+      }
+   
+      close(FILE);
+   
+   }
+   
+   close(STDIN) if (-t STDIN);
+}
+
+# make sure we fork before writing the the pid file ;-)
+
 # We have come this far, let's write the PID to $PIDFILE, useful for
 # killing and stuff.
 
@@ -1628,6 +1769,8 @@ else {
    &syslog("ERRLOG", "cannot write to $LOCKDIR$PIDFILE.$port: $!");
 }
 
+
+
 $SIG{'INT'}='quitdaemonhandler';
 $SIG{'KILL'}='quitdaemonhandler';
 $SIG{'TERM'}='quitdaemonhandler';
@@ -1639,14 +1782,30 @@ $SIG{'TERM'}='quitdaemonhandler';
 # Main waiting loop, wait for connection, and fork of child to process
 # the incoming request
 
+#sub REAPER {
+#    while (waitpid(-1,WNOHANG)) { ; }
+#    $SIG{CHLD} = \&REAPER;  # still loathe sysV
+#}
+#$SIG{CHLD} = \&REAPER;
+
+# reap children when they are done.
+
+use POSIX ":sys_wait_h";
+
+# put limit on loop as sometimes it doesn't stop
+#  - I think it is a bug in perl5.003_7
+$SIG{CHLD}=sub { my $count=0; while (waitpid(-1,WNOHANG) && $count++<10) {} };
+
 for (;!$QUIT;) {
     
-    ($address=accept(NS,S)) || (!$QUIT) || &fatalerror("accept (returncode: \'$address\'): $!");
-    
+    $! = ""; # Clear any previous error codes
+    $address=accept(NS,S);
     last if ($QUIT);
+    next if ( $! =~ /interrupted system call/i );
+    &fatalerror("accept failed: $!") if( ! defined( $address ));
      
     #
-    # fork as savely as possible (see Camel book, fork)
+    # fork as safely as possible (see Camel book, fork)
     
     FORK: {
     
@@ -1655,7 +1814,7 @@ for (;!$QUIT;) {
           #
           # here is the parent process
        
-          while (waitpid(-1, WNOHANGVALUE)>0) {}; 
+#          while (waitpid(-1, WNOHANG)>0) {};   # reap any dead children
       
        }
        elsif (defined($child)) {
@@ -1663,6 +1822,9 @@ for (;!$QUIT;) {
           #
           # this is the child process
           
+	 # close accepting socket
+	 close S;
+
           #
           # exit as cleanly as possible when we do a shutdown
         
@@ -1680,10 +1842,10 @@ for (;!$QUIT;) {
         
           alarm $KEEPOPEN;
         
-          local($af,$port,$inetaddr) = unpack(SOCKADDRVALUE,$address);
-          local($rhost)=join(".",unpack('C4', $inetaddr));
+          local($port,$inetaddr) = sockaddr_in ($address);
+          local($rhost)=inet_ntoa ($inetaddr);
 
-          local($name)=(gethostbyaddr($inetaddr,AF_INETVALUE))[0];
+          local($name)=gethostbyaddr($inetaddr,PF_INET);
         
           #
           # address might not have been reverse delegated...
@@ -1699,7 +1861,7 @@ for (;!$QUIT;) {
           exit 0;
           
        }
-       elsif ($!=~ /No\s+more\s+process/i) {
+       elsif ($!=~ /No\s+more\s+process/i || $! =~ /memory/i) {
        
           #
           # looks like a recoverable fork error, let's try again
